/*
This program inputs a file and all words of that file will be checked 
 whether they are correctly spelled or not. 

  To check the spelling of a particular word, a word-database file is used, 
which includes correct forms of most of the English words. 
First program reads all the words in the database file to a vector,
then it searchs the word-database vector to see if a particular word is there. 

  Program repeats this process for each word of the input file.
Misspelled words are printed to a file. 
*/

#include <iostream>
#include <fstream>
#include <string>
#include "strutils.h"
#include "prompt.h"
#include <vector>

using namespace std;


/*
takes a word and a vector as parameters and searches if the word exists in the vector
if the word exists in the vector, function returns true, otherwise false
Uses binary search method which reads log n of the words. Requires sorted vector
*/
bool checkWordBinary(const vector<string>& spellVector, const string& word) 
{	
    int low = 0;						// leftmost possible entry
    int high = spellVector.size()-1;    // rightmost possible entry
    int mid;							// middle of current range
    while (low <= high)
    {   mid = (low + high)/2;
        if (spellVector[mid] == word)   // found key, exit search
        {   
			return true;
        }
        else if (spellVector[mid] < word)   // key in upper half
        {   
			low = mid + 1;
        }
        else                        // key in lower half
        {   
			high = mid - 1;
        }
    }

	return false;				// not in list
}

/*
takes a word and a vector as parameters and searches if the word exists in the vector
if the word exists in the vector, function returns true, otherwise false
Uses sequential search method which reads all the words in sequential order
*/
bool checkWordSequential(const vector<string>& spellVector, const string& word) 
{	
	int size = spellVector.size();
	
	for (int i = 0; i < size; i++) 
	{	
		string vectorWord = spellVector[i];
		
		if (word == vectorWord) 
		{ 
			return true;
		}
	
	}
	return false;
}


/*
 standard insertion sort for the string vector a
*/
void InsertSort(vector<string> & a, int size)
{
    int k,loc;
    
    for(k=1;k<size;k++) 
    {
        // shift elements to make room for a[k]
        
        string hold = a[k];   // insert this element
        loc = k;            // location for insertion
        while (0 < loc  && hold < a[loc-1])
        {
            a[loc] = a[loc-1];
            loc--;
        }
        a[loc] = hold;
    }
}

/*
read word.txt and place the words in the vector.
*/
bool readAllWords(vector<string>& spellVector) 
{	
	ifstream input;
	input.open("words.dat");
		
	if (input.fail()) 
	{		
		return false;
	}
	string word;
		
	while( getline(input, word) ) 
	{		
		spellVector.push_back(word); //insert into vector via push_back method.
			
	}
		
	input.clear();
	input.close();
	
	return true;

}

int main () 
{	
	string fName, searchType;
	ifstream input;
	
	fName = PromptString("Please Enter the File Name: ");
	input.open(fName.c_str());
		
	if (input.fail())
		return 0;

	searchType = PromptString("Please Choose One of the Search Types (sequential/binary): ");
	
	if(searchType != "binary" && searchType != "sequential")
	{
		cout << "Illegal search type..." << endl;
		return 0;
	}


	string ofName = "spelled_" + fName;
	ofstream output(ofName.c_str());
	
	vector<string> spellVector; //spell vector...
	
	if (!readAllWords(spellVector)) 
	{	
		cout<<"Cannot read dictionary file. Exiting!"<<endl;
	}
	else
	{
		//sort the vector for binary search
		if(searchType == "binary")
		{
			InsertSort(spellVector, spellVector.size());
		}
		output<<"Mispelled words:"<<endl;
		
		string word;
		while ( input >> word) 
		{		
			string originalWord = word;
			StripPunc(word);
			ToLower(word);
			StripPunc(originalWord);
				
			
			if (searchType == "sequential" && !checkWordSequential(spellVector, word)) 
			{		
				output << originalWord << endl;
			}
			
			else if (searchType == "binary" && !checkWordBinary(spellVector, word)) 
			{		
				output << originalWord << endl;
			}
		}
	}
	return 0;
}

