/*
Comparison of sort algorithms...
Bubble Sort
Modified Bubble Sort -->If no exchanges over a pass. Stop processing since sequence is already ordered.
Insertion Sort
Selection Sort

*/
#include <iostream>
#include <string>
#include "ctimer.h"
#include <vector>
#include "sortall.h"
#include "randgen.h"
#include "prompt.h"
using namespace std;

/*
 * Bubble sort algorithm sorts the elements in a vector by making N passes
 * over a vector of N items. On each pass, adjacent elements are compared
 * and if the element on the left (smaller index) is greater it is swapped 
 * with its neighbor. In this manner the largest element bubbles to the end
 * of the vector. On the next pass, adjacent elements are compared again, but
 * the pass stops one short of the end. On each pass, bubbling stops one 
 * position earlier than the pass before until all the elements are sorted.
 */
void bubbleSort(vector<int> &vec, int n) 
{
	int j,k; 
	
	for(j=n-1; j > 0; j--) 
	{	
	// find largest element in 0..k, move to a[j] 
		for(k=0; k < j; k++) 
		{	
			if (vec[k+1] < vec[k]) 
			{
				//Swap(vec[k],vec[k+1]); 
				Swap(vec, k, k+1); 
			} 
		} 
	}	
	
}

/*
 * Bubble sort can be improved by stopping if no values are swapped on some
 * pass, meaning that the elements are in order. This modified version is
 * implementation of this improvement. For ordered or partially ordered sets
 * modified bubble sort performs better than original bubble sort algorithm.
 */
void modifiedBubbleSort(vector<int> &vec, int n) 
{	
	int j,k; 
	bool iterateFlag = true;
	
	for(j=n-1; j > 0 && iterateFlag; j--) 
	{ 
	// find largest element in 0..k, move to a[j] 
		
		iterateFlag = false;
		
		for(k=0; k < j ; k++) 
		{	
			if (vec[k+1] < vec[k]) 
			{
				Swap(vec, k, k+1); 
				iterateFlag = true;

			} 
			
		} 
	}	
	
}


void PrepareVector(vector<int> & a, int count, int mode) 
{ 
    RandGen gen;    // for random # generator 
    int randIndex,k; 
     
    // fill with valuesl 0..count-1 
    if (mode!=3)
	{
		for(k=0; k < count; k++) 
        {
			a[k] = k; 
        }
	}
    else
	{
        for(k=0; k < count; k++) 
        {   
			a[k] = count - 1 - k; 
		}
	}
    // choose random index from k..count-1 and interchange with k 
    if (mode==1)
	{
		for(k=0; k < count - 1; k++) 
        { 
			randIndex = gen.RandInt(k,count-1);   // random index 
            Swap(a,k,randIndex);                  // swap in sortall.h 
        }
	}
} 


int main () 
{	
	int size,minSize,maxSize,incr; // sort minSize, minSize+incr, ... maxSize 
    CTimer timer; 

    cout << "min and max size of vector: "; 
    cin >> minSize >> maxSize; 
    incr =  PromptRange("increment in vector size",1,1000); 
    cout << endl; 
    for (int mode=1; mode<=3; mode++) 
    { 
        if (mode==1) 
            cout << "Random vector" << endl; 
        else if (mode==2) 
            cout << "Ordered vector" << endl; 
        else if (mode==3) 
            cout << "Reverse ordered vector" << endl; 
        cout << "n\tinsert\tselect\tbubble\tmodifiedBubble" << endl; 
        
		for(size=minSize; size <= maxSize; size += incr) 
		{   
			vector<int> copy(size), original(size); 
            cout << size << "\t"; 
            PrepareVector(original,size, mode); 
 
            copy = original;     // sort using insertion sort 
            timer.Start(); 
            InsertSort(copy,copy.size()); 
            timer.Stop(); 
            cout << timer.ElapsedTime() << "\t"; 

            copy = original;     // sort using selection sort 
            timer.Start(); 
            SelectSort(copy,copy.size()); 
            timer.Stop(); 
            cout << timer.ElapsedTime() << "\t"; 

			copy = original;     // sort using bubble sort 
            timer.Start(); 
            bubbleSort(copy,copy.size()); 
            timer.Stop(); 
            cout << timer.ElapsedTime() << "\t"; 

			copy = original;     // sort using modified bubble sort 
            timer.Start(); 
            modifiedBubbleSort(copy,copy.size()); 
            timer.Stop(); 
            cout << timer.ElapsedTime() << "\t\n"; 


        } 
            cout <<"\n\n"; 
	} 
  
	return 0;	
}
